#!/bin/bash

function realpath { echo $(cd $(dirname "$1"); pwd)/$(basename "$1"); }
__FILE__=`realpath "$0"`
__DIR__=`dirname "${__FILE__}"`

BASE_DIR="${__DIR__}"
OUTPUT_DIR="${BASE_DIR}/install-dir"

LIBNAME="libdvbpsi"

SDK_MIN=8.1
CLEAN_BUILD=false

export IPHONEOS_DEPLOYMENT_TARGET=10

if [ -d "${OUTPUT_DIR}" ]; then
    echo "Found ${OUTPUT_DIR}"
else
    mkdir -p ${OUTPUT_DIR}
fi

SRC_DIR="${BASE_DIR}/third-party-src"

ARM_OPT_CFLAGS=""
ARM_OPT_LDFLAGS=""
SIM_OPT_CFLAGS="-O2"
SIM_OPT_LDFLAGS="-O2"

while [ "$#" -gt 0 ]; do
    case $1 in
        --clean)
            CLEAN_BUILD=true
            shift
            continue
            ;;
        --dirty-build)
            CLEAN_BUILD=false
            shift
            continue
            ;;
        --deployment-target)
            if [ "$#" -gt 1 ]; then
                SDK_MIN=$2
                shift 2
                continue
            else
                echo 'ERROR: Must specify a non-empty "--deployment-target" argument.' >&2
                exit 1
            fi
            ;;
        --debug)
            ARM_OPT_CFLAGS="-g -O0"
            ARM_OPT_LDFLAGS="-g -O0"
            SIM_OPT_CFLAGS="-g -O0"
            SIM_OPT_LDFLAGS="-g -O0"
            shift
            continue
            ;;
    esac

    shift
done

function copy_libs () {
    ARCH=${1}

    FROM_DIR="${OUTPUT_DIR}"/lib
    TO_DIR="${OUTPUT_DIR}"/"${ARCH}"
    if [ -d "${TO_DIR}" ]; then
        rm -rf "${TO_DIR}"
    fi
    mkdir -p "${TO_DIR}" &&
    cp -R "${FROM_DIR}"/* "${TO_DIR}/"
}

function _build() {
    pushd . > /dev/null

    ARCH=$1

    if [[ ${CLEAN_BUILD} == true ]]; then
        echo "clean build for ${ARCH}.."
        rm -rf ${SRC_DIR}/${ARCH}
    fi

    if [ ! -d "${SRC_DIR}/${ARCH}" ]; then
        # configure
        CONFIGURE="../configure-iphone --prefix="${OUTPUT_DIR}
    else
        CONFIGURE=""
    fi

    mkdir -p ${SRC_DIR}/${ARCH}

    cd ${SRC_DIR}/${ARCH}

    # flags
    if [[ ! ${CFLAGS} ]]; then
        export CFLAGS="-fembed-bitcode"
    fi
    if [[ ! ${LDFLAGS} ]]; then
        export LDFLAGS=
    fi

    export LDFLAGS="${LDFLAGS} -lstdc++"

    echo "Building for ${ARCH}..."

    echo "./configure for ${ARCH}... with ${CONFIGURE}"
    ARCH="-arch ${ARCH}" ${CONFIGURE}

    echo "make for ${ARCH}..."
    ARCH="-arch ${ARCH}" make

    echo "make install for ${ARCH}..."
    ARCH="-arch ${ARCH}" make install

    copy_libs "$ARCH"
}

function armv7() {
    export LDFLAGS="${ARM_OPT_LDFLAGS} -miphoneos-version-min=$SDK_MIN"
    export CFLAGS="${ARM_OPT_CFLAGS} -miphoneos-version-min=$SDK_MIN"
    _build "armv7"
}
function armv7s() {
    export LDFLAGS="${ARM_OPT_LDFLAGS} -miphoneos-version-min=$SDK_MIN"
    export CFLAGS="${ARM_OPT_CFLAGS} -miphoneos-version-min=$SDK_MIN"
    _build "armv7s"
}
function arm64() {
    export LDFLAGS="${ARM_OPT_LDFLAGS} -miphoneos-version-min=$SDK_MIN"
    export CFLAGS="${ARM_OPT_CFLAGS} -miphoneos-version-min=$SDK_MIN"
    _build "arm64"
}
function i386() {
    export DEVPATH="`xcrun -sdk iphonesimulator --show-sdk-platform-path`/Developer"
    export CFLAGS="${SIM_OPT_CFLAGS} -m32 -mios-simulator-version-min=$SDK_MIN"
    export LDFLAGS="${SIM_OPT_LDFLAGS} -m32 -mios-simulator-version-min=$SDK_MIN"
    _build "i386"
}
function x86_64() {
    export DEVPATH="`xcrun -sdk iphonesimulator --show-sdk-platform-path`/Developer"
    export CFLAGS="${SIM_OPT_CFLAGS} -m64 -mios-simulator-version-min=$SDK_MIN"
    export LDFLAGS="${SIM_OPT_LDFLAGS} -m64 -mios-simulator-version-min=$SDK_MIN"
    _build "x86_64"
}

function lipo() {
    TMP=`mktemp -t lipo`

    INITIAL_ARCH=$1

    TO_DIR="${SRC_DIR}/${LIB_DIR}"
    FROM_DIR="${OUTPUT_DIR}"/"${INITIAL_ARCH}"
    FILE="${FROM_DIR}"/"${LIBNAME}".a
    LIPO_LINE=""
    for ARCH in "$@"; do
        REAL_FILE=`echo ${FILE} | sed -e "s/${INITIAL_ARCH}/${ARCH}/g"`
        LIPO_LINE="${LIPO_LINE} -arch ${ARCH} ${REAL_FILE}"
    done

    OUTPUT_FILE_NAME="${OUTPUT_DIR}"/universal-lib/"${LIBNAME}".a

    mkdir -p "${OUTPUT_DIR}/universal-lib"
    LIPO_LINE="${LIPO_LINE} -create -output ${OUTPUT_FILE_NAME}"
    echo "Lipoing ${OUTPUT_FILE_NAME}..."
    
    xcrun -sdk iphoneos lipo ${LIPO_LINE}
}

cd "${__DIR__}" &&
date "+Starting build at: %c"
echo "Build Directory: $BASE_DIR"
echo "SDK Minimum: $SDK_MIN"

armv7 && arm64 && i386 && x86_64 &&
lipo armv7 arm64 i386 x86_64
date "+Finished build at: %c"
echo "Check $OUTPUT_DIR"


